/*
Copyright (C) 2019 The Imaging Source Europe GmbH
*/

////////////////////////////////////////////////////////////////////////////
// TODO:
// - export@xml2html: index without .js extension at all!!
// - export@xml2html: content data without .htm extension
// - consider '#_sub_page' in exported pages and links
//
////////////////////////////////////////////////////////////////////////////


var current_page;
function json_callback(response) {
    current_page = response;
    $('#json_script').remove();
}

(function (doc_view, $, undefined) {

    // private variables
    var last_page = "";
    var currently_active_tab = "";
    var skip_page_update = false;
    var text_buffer_dict = {} // stores text of all searched pages
    var last_search_text = "" // last search text 
    var last_search_text_elements = [];
    var docs_in_search_list = {}; // already added to search list^^
    var myHistory = []; // back/forward
    var current_index = -1;
    var last_length = 0;

    doc_view.initialize = function () {
        initialize_content();
        initialize_index();
        initialize_splitter();
    }

    doc_view.update_content_page = function (page, callback) {

		// check if page is available in index.
		// If not, it is an external link, or a non-indexed doc page.
		page = page.replace(".htm", ".js")
		var key = index_data.find(d => d.dst === page);
        if (key === undefined) {
			return false;
        }
		
        page = page.replace(".js", "").replace(".htm", "");

        if (last_page !== page) {

            last_page = page;
            load_local_content(page, callback);

            // update history
            if (current_index < myHistory.length) {
                last_length = 0;
                current_index++;
                myHistory[current_index] = page;
            }
            else {
                myHistory.push(page);
                current_index = myHistory.length - 1;
            }
            window.history.pushState(myHistory, "", base_url + '?page=' + page.replace("content/", ""));
			
			return true;
        }
		
		return false;
    }

    doc_view.select_node_by_page = function (page) {
        //var key = index_data.find(d => d.dst === page);
        //if (key !== undefined) {
        select_node(page);
        //}
    }

    doc_view.page_forward = function () {

        current_index++;
        if (current_index >= myHistory.length) {
            current_index = myHistory.length - 1;
            return false;
        }

        var page = myHistory[current_index];
        if (last_page !== page) {
            last_page = page;
            load_local_content(page);
            doc_view.select_node_by_page(page + ".js");
            window.history.replaceState(window.history.state, "", base_url + '?page=' + page.replace("content/", ""));
        }

        return true;
    }

    doc_view.page_back = function () {
        current_index--;
        if (current_index < 0) {
            current_index = 0;
            return false;
        }

        var page = myHistory[current_index];
        if (last_page !== page) {
            last_page = page;
            load_local_content(page);
            doc_view.select_node_by_page(page + ".js");
            window.history.replaceState(window.history.state, "", base_url + '?page=' + page.replace("content/", ""));
        }

        return true;
    }

    doc_view.back_forward = function () {

        stop_search();

        // ceck if back or forward was clicked.
        var dir = -1;
        if (last_length !== 0) {
            dir = Math.sign(window.history.state.length - last_length);
        }
        last_length = window.history.state.length;

        // according to the direction, 
        // execute on custom stack
        if (dir < 0) {
            if (!doc_view.page_back()) {
                window.history.back();
            }
        }
        else {
            if (!doc_view.page_forward()) {
                window.history.forward();
            }
        }
    }


    doc_view.open_tab = function (evt, cityName) {
        currently_active_tab = cityName;
        var i, tabcontent, tablinks;
        tabcontent = document.getElementsByClassName("tabcontent");
        for (i = 0; i < tabcontent.length; i++) {
            tabcontent[i].style.display = "none";
        }
        tablinks = document.getElementsByClassName("tablinks");
        for (i = 0; i < tablinks.length; i++) {
            tablinks[i].className = tablinks[i].className.replace(" active", "");
        }
        document.getElementById(cityName).style.display = "block";
        evt.currentTarget.className += " active";
    }



    function select_node(dst) {
        dst = dst.replace("js", "htm");// content_data still stores with .htm
        var instance = $("#full").jstree(true);
        var branchCont = instance._model.data;
        // deselect all
        for (var branchKey in branchCont) {
            instance.deselect_node(branchKey);
        }
        if (currently_active_tab !== "Content") {
            return;
        }
        // select node with 'text'
        skip_page_update = true;
        for (var branchKey in branchCont) {
            var branch = branchCont[branchKey];
            if (branch.original &&
                branch.original.dst &&
                branch.original.dst.trim() === dst.trim()) {

                instance.select_node(branchKey);
                document.getElementById(branchKey).scrollIntoView();
                break;
            }
        }
    }

    //
    // Search
    //

    function clear_search_box() {
        var selectbox = document.getElementById("searchList");
        for (var i = selectbox.options.length - 1; i >= 0; i--) {
            selectbox.remove(i);
        }
    }

    var quit_last_search = false;
    var is_running = false;
    var timer;

    function start_loading_anim() {
        var tab_Search_elem = document.getElementById("tab_search_input");
        tab_Search_elem.style['background-image'] = "url('spinner.gif')"
    }

    function stop_loading_anim() {
        var tab_Search_elem = document.getElementById("tab_search_input");
        tab_Search_elem.style['background-image'] = "url('searchicon.png')"
    }

    function stop_search() {
        stop_loading_anim();
        quit_last_search = true;
        if (is_running) {
            clearTimeout(timer);
        }
    }

    doc_view.searchbox_search = function () {

        // get search text
        var selectbox = document.getElementById("searchList");
        var searchText = document.getElementById("input_search_id").value;
        // only search if text changed
        if (searchText.length < 3) {
            stop_search();
            clear_search_box();
            docs_in_search_list = {};
        } else if (last_search_text !== searchText) {

            last_search_text = searchText

            clear_search_box();

            // cancel preload
            cancel_preload = true;
            clearTimeout(preload_timer);
            // clear dict that contains already found keys
            docs_in_search_list = {};
            // cancel last search
            stop_search();
            start_loading_anim();
            // start new search
            is_running = true;
            timer = setTimeout(function () {
                quit_last_search = false;
                // extract words from search text
                last_search_text_elements = searchText.toLowerCase().split(/[ ;,]+/);
                for (var j = 0; j < last_search_text_elements.length; j++) {
                    last_search_text_elements[j] = last_search_text_elements[j].replace(/-/g, ' ');
                }
                search_in_files(0);
            }, 250);
        }
    }

    var preload_timer;
    var cancel_preload = false;
    doc_view.preload_search = function () {
        preload_text(0);
    }

    function preload_text(index) {
        if (cancel_preload) {
            return;
        }
        if (index < index_data.length) {
            index++;
            preload_timer = setTimeout(function () {
                var current_file = index_data[index - 1].dst.split("#")[0].trim();
                if (text_buffer_dict[current_file] !== undefined) {
                    // next file
                    preload_text(index);
                } else {
                    get_local_json(current_file, function (obj) {
                        base64_to_texts_dict(current_file, obj.html)
                        // next file
                        preload_text(index);
                    });
                }
            }, 10);
        }
    }

    function base64_to_texts_dict(current_file, base64) {

        // get html from bse64
        var text = atob(base64);

        // convert to UTF-8
        try {
            text = decodeURIComponent(escape(text));
        }
        catch (ex) {
            return;
        }

        // remove html 
        text = strip(text.toLowerCase())
            //extract lines
            .split(/[\r\n/\\]+/).filter(el => el.trim() != "")
            // remove whitespace at start/end of lines
            .map(s => s.trim())
            // combine lines
            .join(" ");

        // add lines to dict
        text_buffer_dict[current_file] = text;

        return text;
    }


    function search_in_files(index) {

        if (quit_last_search) {
            return;
        }

        const index_node = index_data[index];
        const key = index_node.text.trim();
        const current_file = index_node.dst.trim();

        if (current_file.includes("ref_stdfilter")) {
            var h = 0;
        }
        // if text is already available, get content from dict
        if (text_buffer_dict[current_file] !== undefined) {

            const text = text_buffer_dict[current_file];
            search_in_text(last_search_text_elements, text, current_file, key, index);

            index++;
            if (index < index_data.length) {
                search_in_files(index);
            } else {
                is_running = false;
                stop_loading_anim();
            }

        }
        else {
            // else: load json data async and search in callback
            get_local_json(current_file, function (obj) {
                if (obj !== undefined) {

                    const text = base64_to_texts_dict(current_file, obj.html);
                    search_in_text(last_search_text_elements, text, current_file, key, index);

                    index++;
                    if (index < index_data.length) {
                        search_in_files(index);
                    } else {
                        is_running = false;
                        stop_loading_anim();
                    }
                }
            });
        }
    }

    function search_in_text(last_search_text_elements, text, file, key, index) {
        // for each word
        for (var j = 0; j < last_search_text_elements.length; j++) {
            var word = last_search_text_elements[j];
            // for each line
            //for (var i = 0; i < text.length; i++) {
            //if (new RegExp("\\b" + word + "\\b").test(text[i])) {
            if (text.includes(word)) {
                // add to results if not yet added
                if (docs_in_search_list[file] == undefined) {
                    docs_in_search_list[file] = true;
                    var selectbox = document.getElementById("searchList");
                    selectbox.options[selectbox.options.length] = new Option(key, index);
                }
            }
            //}
        }
    }

    doc_view.search_list_clicked = function () {
        var e = document.getElementById("searchList");
        var str = e.options[e.selectedIndex].text;
        for (var i = 0; i < index_data.length; i++) {
            var sp = index_data[i].text.split("#");
            if (index_data[i].text.trim() === str || (sp.length > 1 && sp[1].trim() === str)) {
                doc_view.update_content_page(index_data[i].dst, function () {
                    // highlight text
                    for (var i = 0; i < last_search_text_elements.length; ++i) {
                        $('#myIframe').highlight(last_search_text_elements[i].split(" "));
                    }
                });
                return;
            }
        }
    }

    //
    // Index
    //

    function initialize_index() {
        index_data.sort(function (a, b) {
            if (a.text < b.text) { return -1; }
            if (a.text > b.text) { return 1; }
            return 0;
        })

        var select = document.getElementById("indexList");
        for (var i = 0; i < index_data.length; i++) {
            select.options[select.options.length] = new Option(index_data[i].text, i);
        }
    }

    doc_view.index_search = function () {
        var text = document.getElementById("search").value;
        var len = text.length;
        for (var l = len; l > 0; l--) {
            var subStr = text.substr(0, l);
            for (var i = 0; i < index_data.length; i++) {
                var txt = index_data[i].text;
                if (txt.toLowerCase().startsWith(subStr.toLowerCase())) {
                    doc_view.update_content_page(index_data[i].dst);
                    var element = document.getElementById("indexList");
                    element.value = i;
                    return;
                }
            }
        }
    }

    doc_view.index_clicked = function () {
        var x = document.getElementById("indexList").value;
        if (x >= 0 && x < index_data.length) {
            doc_view.update_content_page(index_data[x].dst);
        }
    }


    //
    // Tree
    //
    function initialize_content() {
        $('#full').jstree({
            'core': {
                "themes": { "dots": false },
                'data': content_data,
                "rtl": true,
                "animation": 0
            },
            "plugins": ["wholerow"]
        });
        $('#full')
            .on('changed.jstree', function (e, data) {
                if (skip_page_update) {
                    skip_page_update = false;
                    return;
                }
                var i, j, r = [];
                for (i = 0, j = data.selected.length; i < j; i++) {
                    r.push(data.instance.get_node(data.selected[i]).text);
                }
                if (typeof data.node !== "undefined") {
                    if (typeof data.node.original.dst === "undefined") {
                        last_page = data.node.original.text;
                        alert("Page does not exist: " + r.join(', '));
                    }
                    else if (data.selected.length > 0) { // only change page, if selected
                        doc_view.update_content_page(data.node.original.dst);
                    }
                }
            })
            .jstree();
    }


    //
    // Splitter
    //
    var select_none = "-webkit-user-select: none; -moz-user-select: none; -ms-user-select: none;user-select: none; ";
    var select_default = "-webkit-user-select: default; -moz-user-select: default; -ms-user-select: default;user-select: default; ";
    var iframe_layout = "padding-left:10px; padding-right:10px; width:calc(100%-10px); height:100%; overflow-y:auto; ";
    var last_page_x = -1;
    function initialize_splitter() {
        var leftPane = document.getElementById('left-pane');
        var rightPane = document.getElementById('right-pane');
        var paneSep = document.getElementById('panes-separator');
        var leftLimit = 10;
        var rightLimit = 90;
        var originalIFrameStyle = document.getElementById('myIframe').style;
        //last_page_x = leftPane.clientWidth;


        paneSep.sdrag(function (el, pageX, startX, pageY, startY, fix) {

            var iframe = document.getElementById('myIframe');
            iframe.style = "pointer-events: none; " + select_none + iframe_layout;

            fix.skipX = true;
            last_page_x = pageX;

            if (pageX < window.innerWidth * leftLimit / 100) {
                pageX = window.innerWidth * leftLimit / 100;
                fix.pageX = pageX;
            }
            if (pageX > window.innerWidth * rightLimit / 100) {
                pageX = window.innerWidth * rightLimit / 100;
                fix.pageX = pageX;
            }

            var cur = pageX / window.innerWidth * 100;
            if (cur < 0) {
                cur = 0;
            }
            if (cur > window.innerWidth) {
                cur = window.innerWidth;
            }
            var right = (100 - cur - 2);
            leftPane.style.width = cur + '%';
            rightPane.style.width = right + '%';

        }, function () {
            var iframe = document.getElementById('myIframe');
            iframe.style = "pointer-events: auto; " + select_default + iframe_layout;
        }, 'horizontal');
    }

    // 
    // async json load 
    // 
    function get_local_json(json_url, callback) {
        var json_script = document.createElement('script');
        json_script.type = 'text/javascript';
        json_script.charset = "utf-8";
        json_script.async = false;
        if (json_script.readyState) {  // only required for IE <9
            script.onreadystatechange = function () {
                if (json_script.readyState === "loaded" || json_script.readyState === "complete") {
                    json_script.onreadystatechange = null;
                    callback(current_page);
                }
            };
        } else {  //Others
            json_script.onload = function () {
                callback(current_page);
            };
        }
        json_script.src = json_url;
        json_script.id = 'json_script';
        document.getElementsByTagName('head')[0].appendChild(json_script);
    }

    function load_local_content(current_file, callback) {

        // load json data async and search in callback
        get_local_json(current_file + ".js", function (obj) {
            if (obj !== undefined) {
                var text = atob(obj.html);
                var c = document.getElementById("myIframe");
                c.innerHTML = decodeURIComponent(escape(text));

                // for some reason right-pane's width in changed, when the 
                // content changed...
                var leftPane = document.getElementById('left-pane');
                var rightPane = document.getElementById('right-pane');
                var paneSep = document.getElementById('panes-separator');
                if (last_page_x < 0) {
                    last_page_x = Math.max(350, window.innerWidth * 0.3);
                }
                var cur = last_page_x / window.innerWidth * 100;
                if (cur < 0) {
                    cur = 0;
                }
                if (cur > window.innerWidth) {
                    cur = window.innerWidth;
                }
                var right = (100 - cur - 2);
                leftPane.style.width = cur + '%';
                rightPane.style.width = right + '%';
                var iframe = document.getElementById('myIframe');
                iframe.style = "pointer-events: auto; " + select_default + iframe_layout;

                if (callback !== undefined) {
                    callback();
                }
            }
        });
    }

    function getPageText() {
        return $('html')[0].outerHTML
    }

    function strip(html) {
        var tmp = document.createElement("DIV");
        tmp.innerHTML = html;
        return tmp.textContent || tmp.innerText || "";
    }

}(window.doc_view = window.doc_view || {}, jQuery));