
#ifndef TIS_BARCODE_H_INC_
#define TIS_BARCODE_H_INC_

#if !defined IC_BARCODE_API
#ifdef TIS_BARCODE_EXPORTS
	#if defined(_MSC_VER)
		#define IC_BARCODE_API __declspec(dllexport)
	#elif defined(__GNUC__)
		#define IC_BARCODE_API __attribute__((visibility("default")))
	#endif
#else
	#if defined(_MSC_VER)
		#define IC_BARCODE_API __declspec(dllimport)
	#elif defined(__GNUC__)
		#define IC_BARCODE_API
	#endif
#endif
#endif


#include "ic_barcode_enums.h"

typedef enum ICBarcode_Error
{
    ICBarcode_SUCCESS = 0,
    ICBarcode_INVALID_ARGUMENT = -1,
    ICBarcode_PARAM_NOT_SET = -2,
    ICBarcode_NO_LICENSE = -3,
} ICBarcode_Error;

struct ICBarcode_Scanner;

typedef struct ICBarcode_PointF
{
	float x, y;
} ICBarcode_PointF;


typedef struct ICBarcode_Result
{
	char Text[4096];
	char RawBytes[4096];
	int RawBytesLength;
	float Frequency;
    ICBarcode_PointF ResultPoints[4];
	int ResultPointsLength;
	int Format;
} ICBarcode_Result;

extern "C"
{
    IC_BARCODE_API ICBarcode_Scanner* ICBarcode_CreateScanner();
    IC_BARCODE_API void                ICBarcode_DestroyScanner( ICBarcode_Scanner* ctx );
    IC_BARCODE_API int                 ICBarcode_SetBarcodeFormats( ICBarcode_Scanner* ctx, int formats );
    IC_BARCODE_API int                 ICBarcode_GetBarcodeFormats( ICBarcode_Scanner* ctx, int* formats );
    IC_BARCODE_API int                 ICBarcode_ResetAllParams( ICBarcode_Scanner* ctx );
    IC_BARCODE_API int                 ICBarcode_SetParam( ICBarcode_Scanner* ctx, ICBarcode_Param param, int value );
    IC_BARCODE_API int                 ICBarcode_GetParam( ICBarcode_Scanner* ctx, ICBarcode_Param param, int* value );
    IC_BARCODE_API int                 ICBarcode_SetFormatParam( ICBarcode_Scanner* ctx, ICBarcode_Format format, ICBarcode_Param param, int value );
    IC_BARCODE_API int                 ICBarcode_GetFormatParam( ICBarcode_Scanner* ctx, ICBarcode_Format format, ICBarcode_Param param, int* value );
    IC_BARCODE_API int                 ICBarcode_FindBarcodes( ICBarcode_Scanner* context, unsigned char* src, int width, int height, int stride, ICBarcode_Result* result, int maxBarcodeResults );
	IC_BARCODE_API int					ICBarcode_FindBarcodesHardcore(ICBarcode_Scanner* context, unsigned char* src, int width, int height, int stride, ICBarcode_Result* result, int maxBarcodeResults);
    IC_BARCODE_API void                ICBarcode_Transform_BGRA32_to_Y800( unsigned char* dstPtr, unsigned char* srcPtr, int width, int height, int src_stride, int dst_Stride );
    IC_BARCODE_API void                ICBarcode_Transform_BGRA24_to_Y800( unsigned char* dstPtr, unsigned char* srcPtr, int width, int height, int src_stride, int dst_Stride );
}


#endif // TIS_BARCODE_H_INC_