#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QPushButton>
#include <QGridLayout>
#include <QtGui>
#include <QTextEdit>

#include "tcamcamera.h"
#include "ic_barcode.h"

// Define your custom event identifier
const QEvent::Type BARCODE_FOUND_EVENT = static_cast<QEvent::Type>(QEvent::User + 1);

// Define your custom event subclass
class BarcodeFoundEvent : public QEvent
{
    public:
        BarcodeFoundEvent(const int customData1, const int customData2):
            QEvent(BARCODE_FOUND_EVENT),
            m_customData1(customData1),
            m_customData2(customData2)
        {
        }

        int getCustomData1() const
        {
            return m_customData1;
        }

        int getCustomData2() const
        {
            return m_customData2;
        }

    private:
        int m_customData1;
        int m_customData2;
};

////////////////////////////////////////////////////////////////////////


class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = nullptr);
    void postBarcodeFoundEvent(const int customData1, const int customData2);
    ~MainWindow();

protected:
    void customEvent(QEvent *event);

private:
    void OnSelectDevice();
    void OnDeviceProperties();
    void SaveLastUsedDevice(std::string serial, int width, int height, int fps1, int fps2);
    bool LoadLastUsedDevice();
    void OpenandStartDevice(std::string serial, int width, int height, int fps1, int fps2);


    void handleBarcodeFoundEvent(const BarcodeFoundEvent *event);
    static GstFlowReturn new_frame_cb(GstAppSink *appsink, gpointer data);

private:
    QWidget *window;
    QHBoxLayout *mainLayout;
    QWidget* _VideoWidget;
    QMenu *_fileMenu;
    QMenu *_deviceMenu;
    QAction *_exitAct;
    QAction *_DeviceSelectAct;
    QAction *_DevicePropertiesAct;
    gsttcam::TcamCamera *_pCam;
    QTextEdit *_BarcodeList;

    typedef struct
    {
        int ImageCounter;
        bool busy;
        MainWindow *pMainWindow;
        ICBarcode_Scanner* pIC_BarcodeScanner;
        int FoundBarcodes;
        std::vector<ICBarcode_Result> BarCodes;
        
    } CUSTOMDATA;

    CUSTOMDATA _CustomData;

};

#endif // MAINWINDOW_H
